; All these sources/objects are (c) Simon Brattel 1992

		include ..\symbols.inc	; Define them
		include gsymbols.inc	;

CODE		SEGMENT public 		;

		ASSUME  CS:CODE		;

; Filenames

sSource		db "e:\dpaint2\pictures\blocks.lbm",0	; Blocks
sDest		db "s_salad1.bl0",0	; Dest

Start:		mov ax,StackSeg		; get Stack Segment Origin
		mov SS,ax		; and initialize the SS register
		mov SP,o TOS		; set SP to Top Of Stack

		; Free surplus RAM

		call Deallocate		; Release it

		; Now, read the source file

		call Read_Source	; Get it
		jc Exit			; Fail

		; Now, we have the file, convert it

		call Convert		; Convert it

		; Now, save it

		call Write_Dest		; Write it

	; The exit message

Exit:		mov ah,terminate	;
		int dos			; Done

; Convert the file

Convert:	mov ax,DATA1		; The file
		mov es,ax		;

		mov ax,DATA		; The output data
		mov ds,ax		;

		; Now, get the palette

		mov si,0030h+o File + iCol*3 ; HACK ! this is the start of the CMAP
		mov di,o Blocks		; Point at the first block

		mov cx,nCols*3		; This many colours

Conv_LpC:	mov al,es:[si]		; Giz the value
		inc si			;

		shr al,1		; / 4
		shr al,1		;

		mov ds:[di],al		; Store it
		inc di			;

		loop Conv_LpC		; Loop

		; Now convert the graphics

		mov cs:pBlocks,di	; Save the ptr

		mov si,03E8h+o File	; HACK ! this is the start of the BODY

		; 8 lines

		mov cx,8		; This many

Conv_Lp:	push cx			; Save

		; Now read one line of blocks

		call Read_Blocks	; Read 16 blocks

		; Now, write this to the file

		push si			; Save the ptr
		call Write_Blocks	; Write 16 blocks
		pop si			;

		; Loop

		pop cx			; Restore
		loop Conv_Lp		; Loop

		ret			; Done

	; Now, write one line of blocks

Write_Blocks:	mov ax,DATA		; The data-area
		mov ds,ax		;

		mov di,cs:pBlocks	; Point at the block data
		mov si,o Pixel_Buffer	; Point at the graphics

		mov cx,16		; Convert this many blocks

WB_Lp:		push cx			; Save
		push si			;
		push di			;

		call Write_Block	; Write one

		pop di			; Restore
		pop si			;
		pop cx			;

		lea di,256[di]		; Skip this block
		lea si,17[si]		;		

		loop WB_Lp		; Loop

		; Step the ptr

		mov cs:pBlocks,di	; Store ptr

		ret			; Done

	; Now, write one block

Write_Block:	mov cx,16		; Copy this many lines

WBL_LpY:	mov dx,16		; Copy this many pixels

WBL_LpX:	mov al,ds:[si]		; Next pixel
		inc si			;
		mov ds:[di],al		;
		inc di			;

		dec dx			; Loop
		jnz WBL_LpX		;

		; Move to the next line

		lea si,320-16[si]	; Move down

		dec cx			; Loop
		jnz WBL_LpY		;

		ret			; Done

	; Now, read one line of blocks

Read_Blocks:	mov ax,DATA		; The data-area
		mov ds,ax		;

		mov di,o Pixel_Buffer	; Point at the buffer

		mov cx,17		; Read this many lines

RB_Lp1:		push cx			; Save

		call Read_Line		; Read a pixel line

		pop cx			; Restore
		loop RB_Lp1		; Loop

		ret			; Done

; Read a pixel-line

Read_Line:	lea bp,320[di]		; Point at the end address

		mov dl,01h		; This bit
		mov dh,00h		; Mask

RL_Lp:		push di			; Read a block
		call Read_Line_Bit	; Read a bit
		pop di			;

		mov dh,0FFh		; New mask

		add dl,dl		; Shift the bit
		jnc RL_Lp		; Loop

		; Skip this line

		mov di,bp		; Skip to the next line

		ret			; Done

	; Read a line of bits

RLB_Skip:	inc si			; Skip

Read_Line_Bit:	mov al,es:[si]		; Get the next byte of the file
		inc si			; Skip

		cmp al,-127		; Ignore ?
		je RLB_Skip		; Yes, loop

		test al,080h		; -ve ?
		jnz RLB_Neg		; Yes, skip

		; Copy the next (n+1) bytes

		inc al			; n+1

RLB_C_Lp:	mov ah,es:[si]		; Copy a byte
		inc si			;

		call RLB_Write		; Write 8 bits

		dec al			; Loop
		jnz RLB_C_Lp		;

		; End of line ?

RLB_EOL:	cmp di,bp		; EOL ?
		jnz Read_Line_Bit	; No, loop

		ret			; Done

	; Copy the next byte (n+1) times

RLB_Neg:	neg al			; Negate it
		inc al			; n+1

		mov ah,es:[si]		; Copy a byte
		inc si			;

RLB_D_Lp:	call RLB_Write		; Write 8 bits

		dec al			; Loop
		jnz RLB_D_Lp		;

		jmp s RLB_EOL		; Skip

	; Write 8 bits

RLB_Write:	push ax			; Save

		mov al,8		; This many bits

RLBW_Lp:	mov bl,ds:[di]		; Mask it
		and bl,dh		;

		add ah,ah		; Test the top bit
		jnc RLBW_Z		; No, skip

		or bl,dl		; Merge it		

RLBW_Z:		mov ds:[di],bl		; Write it
		inc di			; Skip

		dec al			; Loop
		jnz RLBW_Lp		;

		pop ax			; Restore
		ret			; Done

; Read the source file

Read_Source:	mov dx,o sSource	; Get the ^string

		push cs			; ds:dx
		pop ds			;

		mov bx,o File		; Point at the map
		mov cx,8		; Length

		mov ax,DATA1		; The segment
		mov es,ax		;

		call Read_File		; Read it
		jc RF_Exit		; Can't, exit

		; Now, get the length

		mov bx,o File + 6	; Length
		mov cx,es:[bx]		; Get the length
		xchg cl,ch		; Order them

		add cx,8		; Length (total)

		; Now read the file

		mov dx,o sSource	; Get the ^string

		push cs			; ds:dx
		pop ds			;

		mov bx,o File		; Point at the map

		mov ax,DATA1		; The segment
		mov es,ax		;

		call Read_File		; Read it
		jc RF_Exit		; Can't, exit

		clc			; Ok

RF_Exit:	ret			; Done

; Write the destn file

Write_Dest:	mov dx,o sDest		; Get the ^string

		push cs			; ds:dx
		pop ds			;

		mov bx,o Blocks		; Point at the map
		mov cx,16*16*16*8 + nCols*3 ; Length

		mov ax,DATA		; The segment
		mov es,ax		;

		call Write_File		; Read it

		ret			; Done

; Possible read filename dx:ds, length cx, data from es:bx

Read_File:	mov cs:WF_Len,cx	; Save the length

		mov cs:WF_Off,bx	; Start of data

		mov ax,es		; Save the seg
		mov cs:WF_Seg,ax	;

		mov cx,00h		; File attribute

		mov al,0		; Read only

		mov ah,OpenFile		; Call dos
		int dos			;

		jc PRF_Exit		; Can't open file

		mov cs:TempHandle,ax	; Save the handle

		; Rewind to start

		mov cx,0		; Rewind file
		mov dx,0		;

		mov bx,ax		; Handle

		mov al,0		;

		mov ah,MovePointer	;
		int dos			;

		; Read the file

		mov ax,cs:WF_Seg	; Segment
		mov ds,ax		;

		mov bx,cs:WF_Off	; Offset

		mov cx,cs:WF_Len	; Length

		call Read_Block		; Write it

		; Close the file

		push ax			; Save error code

		mov bx,cs:TempHandle	; Close it
		mov ah,CloseFile	;
		int dos			;

		pop ax			;

		; Done

PRF_Exit:	ret			; Done

; Write filename dx:ds, length cx, data from es:bx

Write_File:	mov cs:WF_Len,cx	; Save the length

		mov cs:WF_Off,bx	; Start of data

		mov ax,es		; Save the seg
		mov cs:WF_Seg,ax	;

		mov cx,00h		; File attribute

		mov ah,CreateFile	; Call dos
		int dos			;

		jc WF_Fail		; Can't create file

		mov cs:TempHandle,ax	; Save the handle

		; Write the keys

		mov ax,cs:WF_Seg	; Segment
		mov ds,ax		;

		mov bx,cs:WF_Off	; Offset

		mov cx,cs:WF_Len	; Length

		call Save_Block		; Write it

		; Close the file

		push ax			; Save error code

		mov bx,cs:TempHandle	; Close it
		mov ah,CloseFile	;
		int dos			;

		pop ax			;

		; Done

WF_Fail:	ret			; Done

; Write block ds:bx for length cx

Save_Block:	push ax			; Save reg
		push dx			;

		mov dx,bx		; Get ptr

		mov bx,cs:TempHandle	; Get file handle

		mov ah,WriteToFile	; Write it
		int dos			;
		jc SB_Fail		; Fail

		cmp ax,cx		; Save full length ?
		jz SB_OK		; Yes, skip

		stc			; Assert fail

SB_Fail:	pop dx			; Restore
		pop ax			; dummy

		mov al,64		; "Disc full"

		ret			;

SB_OK:		pop dx			; Restore
		pop ax			;

		ret			;

; Read block ds:bx for length cx

Read_Block:	push ax			; Save reg
		push dx			;

		mov dx,bx		; Get ptr

		mov bx,cs:TempHandle	; Get file handle

		mov ah,ReadFromFile	; Read it
		int dos			;
		jc SB_Fail		; Fail

		cmp ax,cx		; Save full length ?
		jz SB_OK		; Yes, skip

		stc			; Assert fail

		jmp SB_Fail		; Fail

; Vars

WF_Len		dw ?			; Length
WF_Seg		dw ?			; Segment
WF_Off		dw ?			; Offset

TempHandle	dw ?			; Handle

; Release the unused RAM

Deallocate:	mov bx,o TOS		; Point at top of ss

		mov cl,4		; Convert to paragraphs
		shr bx,cl		;
		inc bx			; add one for luck

		mov cx,ss		; Get stack segment
		add bx,cx		; Add stack segment
		
		mov cx,es		; Get extra segment
		sub bx,cx		; Subtract it

		; bx contains no. of paragraphs from es to the top of ss

		mov ah,Set_Block	; Deallocate surplus RAM
		int dos			;

		mov al,9		; "Internal memory"
		ret			;

; Vars

pBlocks		dw ?			; Pointer

CODE		ends

; Data

DATA		SEGMENT byte public	; A segment for the fonts

Blocks		db (16*16*16*8) + (nCols*3) dup (0)	; Build them into here

Pixel_Buffer	db 320*17 dup (?)	; Screen buffer

DATA		ends			; End it

DATA1		SEGMENT byte public	; A segment for the file

File		db 65535 dup (?)	; Load it into here

DATA1		ends			; End it

StackSeg	segment stack

		dw 400 dup (?)
TOS		equ this word

StackSeg	ends

		end Start

