; Drivers for the screen

; Update the screen from the current buffer, and swap the active buffer

Update_Screen:	cli			; Kill the interrupts

		mov ax,ss		; Save the stack
		mov cs:Save_SS,ax	;
		mov cs:Save_SP,sp	;

		; Setup the segments

		mov ax,cs:Buff_1_Seg	; Buffer 1
		mov bx,cs:Buff_2_Seg	; Buffer 2

		; Which way round ?

		test b cs:Curr_Buff_F,1	; Buffer 1 or 2 current ?
		jz US_1			; Buffer 1 = current

		xchg ax,bx		; Swap

US_1:		mov ss,ax		; Use ss for the active one
		mov ds,bx		; Use ds for the old one

		mov ax,0A000h		; VDU seg
		mov es,ax		; In ES

		; Now, enable the stack again

		mov sp,Buff_Offset	; Offset to the first used byte
		sti			; Enable ints again

		; Now, we copy any bits that have changed

		mov si,Buff_Offset	; Start of graphics

		mov di,View_Offset	; Start of window

		; Now, loop across

		mov cx,View_Height	; This many lines

US_LpY:		mov dx,View_Width/4	; This many pixels/4

		jmp s US_LpX		; align

		align 16

US_LpX:		mov eax,ss:[si]		; Get the current pixel
		mov ebx,ds:[si]		; Old value
		lea si,4[si]		; Skip

		cmp eax,ebx		; Same ?
		jnz US_Change		; No, skip

US_NoChange:	lea di,4[di]		; Skip destn.

		dec dx			; Loop across
		jnz US_LpX		;

		; Now, skip to the next line

		add si,Margin_X		; Skip the margin
		add di,320-View_Width	;

		dec cx			; Loop in Y
		jnz US_LpY		;

		; Now, we must recover the stack again

US_Exit:	cli			; Disable ints

		mov ax,cs:Save_SS	; SS
		mov ss,ax		;

		mov ax,cs:Save_SP	; SP
		mov sp,ax		;

		sti			; Allow ints again

		; Now, swap the buffers over

Swap_Pages:	mov al,cs:Curr_Buff_F	; Get the flag
		inc al			; Swap it
		and al,1		;
		mov cs:Curr_Buff_F,al	;

		; Calc the current segment

		mov bx,cs:Buff_1_Seg	;

		test al,1		; Which is current ?
		jz US_Swap		; 1 is current

		mov bx,cs:Buff_2_Seg	;

US_Swap:	mov cs:Buff_Seg,bx	; Use this one

		ret			; Done

	; Now we have the same loop, but defaulting to the changed case

		align 16

US_Chg_LpX:	mov eax,ss:[si]		; Get the current pixel
		mov ebx,ds:[si]		; Old value
		lea si,4[si]		; Skip

		cmp eax,ebx		; Same ?
		jz US_NoChange		; No, skip

		; The pixel is different, show it

US_Change:	mov es:[di],eax		; Write to the display

		lea di,4[di]		; Skip

		dec dx			; Loop across
		jnz US_Chg_LpX		;

		; Now, skip to the next line

		add si,Margin_X		; Skip the margin
		add di,320-View_Width	;

		dec cx			; Loop in Y
		jnz US_LpY		;

		; Done, exit

		jmp s US_Exit		; As above

; Get the display buffers

Get_Buff_RAM:	mov bx,Buffer_Size_P	; Size in segments

		; Allocate this much

		mov ah,Allocate_Mem	; Call DOS
		int dos			;
		jc GBR_X		; Exit

		; Save the address

		mov cs:Buff_1_Seg,ax	; Save it
		mov cs:Buff_Seg,ax	;

		mov bx,Buffer_Size_P	; Size in segments

		; Allocate this much

		mov ah,Allocate_Mem	; Call DOS
		int dos			;
		jc GBR_X		; Exit

		; Save the address

		mov cs:Buff_2_Seg,ax	; Save it

GBR_X:		ret			; Done

; Setup the buffers

Init_Display:	mov ax,0A000h		; VDU Segment
		mov ds,ax		;

		; CLS

		call Setup_Tartan	; Background

		mov di,o dDisp_Defn	; Display Defn
		call Draw_List		; Draw it

		; Draw the game screen

		mov dx,o sScreen_Name	; Name of the screen file
		call Draw_File		; Draw it

		; Now, clear the 2 buffers

		mov ds,cs:Buff_1_Seg	; Point at it

		call Clear_Buff		; Clear it

		mov ds,cs:Buff_2_Seg	; Point at it

		call Clear_Buff		; Clear it

		; Now, assert drawing into buffer 1

		mov b cs:Curr_Buff_F,0	; Z
		mov ax,cs:Buff_1_Seg	; Seg of buffer 1
		mov cs:Buff_Seg,ax	;

		; Now, show the status areas

		call Setup_Status	; Show them

		; Done, exit

		ret			; Done

; Draw a file to the screen

Draw_File:	call Open_File		; Open the file
		jc DFi_X		; Can't, exit

		mov ax,0A000h		; VDU Segment
		mov ds,ax		;

		; Read it to the display

		mov di,0		; To here

DFi_Lp:		call Get_File_Char	; Get the next byte
		jc DFi_X		; Failed, exit

		; This is a count

		cmp al,0		; Repeat ?
		js DFi_RptByte		; Rep
		je DFi_X		; End

		; This is a copy count

		mov cl,al		; Save

DFi_Lp1:	call Get_File_Char	; Get it

		cmp al,0		; Ignore byte ?
		je DFi_Skip1		; Yes, skip

		mov [di],al		; Store

DFi_Skip1:	inc di			;

		dec cl			; Loop
		jne DFi_Lp1		;

		jmp s DFi_Lp		; Loop

	; Repeat a byte

DFi_RptByte:	neg al			; Calc count
		mov cl,al		; Save

		call Get_File_Char	; Get the next byte

		cmp al,0		; Ignore ?
		je DFi_Skip2		; Yes, skip

DFi_Lp2:	mov [di],al		; Save it
		inc di			;

		dec cl			; Loop
		jnz DFi_Lp2		;

		jmp s DFi_Lp		; Loop

DFi_Skip2:	mov ch,0		; 16-bit
		add di,cx		; Ignore it

		jmp s DFi_Lp		; Loop

	; Done, exit

DFi_X:		ret			; Done

; Fill the background of the display

Setup_Tartan:	mov bl,20h		; Colour 00

		mov ax,0A000h		; VDU Segment
		mov ds,ax		;

		mov di,0		; Point at top-left

		mov dx,200		; Lines down

ST_Y_Lp:	mov cx,320		; X loop

		add bl,8		; Step down

		mov bh,bl		; X cnt
		and bh,07h		;

ST_X_Lp:	mov al,bh		; X
		and al,07h		;

		mov ah,bl		;
		and ah,38h		;
		add al,ah		;
		and al,3Fh		; Mask
		add al,0C0h		; Offset

		mov [di],al		; Set it

ST_Skip:	inc bh			; Next

		inc di			; Step ptr

		mov [di],al

;		inc di			; Skip pixel

		dec cx			; X Loop
		jnz ST_X_Lp		;

;		lea di,320[di]		; Skip line

		dec dx			; Y-loop
		jnz ST_Y_Lp		;

		ret			; Done

; Clear a buffer (DS) to black

Clear_Buff:	mov di,Buff_Offset	; Start of buffer
		mov cx,Buff_Size	; Size

		mov al,0		; Clear to 00

CB_Lp:		mov ds:[di],al		; Clear it
		inc di			;

		dec cx			; Loop
		jnz CB_Lp		;

		ret			; Done

; Given a list of drawing commands, draw the main screen

Draw_List:	mov ax,0A000h		; VDU Seg
		mov ds,ax		;

DL_Lp:		mov ax,cs:[di]		; Giz a command/Colour

		cmp al,"R"		; Rectangle ?
		je DL_Rect		; Yes, skip

		cmp al,"B"		; Box ?
		je DL_Box		; Yes, skip

		ret			; Done

	; Draw a rectangle

DL_Rect:	push ax			; Save colour

		; Drive X

		mov si,cs:2[di]		; X
		mov cx,cs:6[di]		; X1
		sub cx,si		; dX
		inc cx			; dX+1

		; Drive Y

		mov ax,cs:4[di]		; Y
		mov dx,0		;

		mov bx,320		; Y * 320
		mul bx			; * 320

		add si,ax		; Calc address

		mov dx,cs:8[di]		; Y1
		sub dx,cs:4[di]		; dY
		inc dx			; dY+1

		pop ax			; Restore colour

		lea di,10[di]		; Skip params

DLR_Lp:		push cx			; Save X step
		push si			; Save X

DLRL_Lp2:	mov ds:[si],ah		; Write it
		inc si			; Step

		dec cx			; Loop
		jnz DLRL_Lp2		;

		pop si			; Restore
		pop cx			;

		; Y loop

		add si,320		; Next line

		dec dx			; Loop
		jnz DLR_Lp		;

		; Done

		jmp s DL_Lp		; Loop

	; Draw a box

DL_Box:		push ax			; Save col

		mov si,cs:2[di]		; X

		mov ax,cs:4[di]		; Y
		mov dx,0		;
		mov bx,320		;

		mul bx			;

		add si,ax		; Calc start point

		pop ax			; Restore colour

		; Right

		mov cx,cs:6[di]		; X1
		sub cx,cs:2[di]		; dX

		mov dx,1		; Offset

		push cx			; Save X count

		call DLL_Lp		; Draw it

		; Down

		mov cx,cs:8[di]		; Y1
		sub cx,cs:4[di]		; dY

		mov dx,320		; Offset

		call DLL_Lp		; Draw it

		; Left
		
		pop cx			; Get X count

		mov dx,-1		; Offset

		call DLL_Lp		; Draw it

		; Up

		mov cx,cs:8[di]		; Y1
		sub cx,cs:4[di]		; dY

		mov dx,-320		; Offset

		call DLL_Lp		; Draw it

		lea di,10[di]		; Skip command

		jmp DL_Lp		; Loop

	; Draw a line

DLL_Lp:		mov ds:[si],ah		; Draw it
		add si,dx		; Move

		dec cx			; Loop
		jnz DLL_Lp		;

		ret			; Done

; Fixed data

dDisp_Defn	equ this byte

;		db "R",07h		; Background
;		dw 0,0,320,200		; X,Y,X1,Y1

		db "R",00h		;
		dw View_LHS,View_TOP	;
		dw View_RHS,View_BOTT	;

		; Border

;		db "B",04h		;
;		dw View_LHS-1,View_TOP-1;
;		dw View_RHS+1,View_BOTT+1;

;		db "B",06h		;
;		dw View_LHS-2,View_TOP-2;
;		dw View_RHS+2,View_BOTT+2;

;		db "B",05h		;
;		dw View_LHS-3,View_TOP-3;
;		dw View_RHS+3,View_BOTT+3;

;		db "B",02h		;
;		dw View_LHS-2,View_TOP-2;
;		dw View_RHS+2,View_BOTT+2;

;		db "R",7
;		dw View_LHS,View_TOP
;		dw View_LHS+47,View_TOP+47

		; End

		db 0			; End

		db "R",03h		; Background
		dw 0,0,320,200		; X,Y,X1,Y1

		; Window

	ifndef flat_files

sScreen_Name	db "GAMECODE\S_SALAD1.TI0",0	; Filename

	else

sScreen_Name	db "S_SALAD1.TI0",0		; Filename

	endif

; Vars

Curr_Buff_F	db 0			; Z = 1, NZ = 2
Buff_1_Seg	dw 0			; Segment of drawing buffer
Buff_2_Seg	dw 0			; Segment of drawing buffer

